////////////////////////////////////////////////////////////////////////////////
//Copyright (c) 2005
//	Lukasz Matecki
//	lutecki@op.pl
////////////////////////////////////////////////////////////////////////////////
//April 25, 2006 08:57:43 AM 
////////////////////////////////////////////////////////////////////////////////
#include "Main.h"

#define UART_RX_BUFFER_MASK ( UART_RX_BUFFER_SIZE - 1 )
#if ( UART_RX_BUFFER_SIZE & UART_RX_BUFFER_MASK )
   #error RX buffer size is not a power of 2
#endif

#define UART_TX_BUFFER_MASK ( UART_TX_BUFFER_SIZE - 1 )
#if ( UART_TX_BUFFER_SIZE & UART_TX_BUFFER_MASK )
   #error TX buffer size is not a power of 2
#endif
////////////////////////////////////////////////////////////////////////////////
/* Static Variables */
static unsigned char UART_RxBuf[UART_RX_BUFFER_SIZE];
static unsigned char UART_TxBuf[UART_TX_BUFFER_SIZE];
////////////////////////////////////////////////////////////////////////////////

extern void UART_init(void)//inicjalizacja komunikacji
{
unsigned char x;
   
	//UBRRH=0x01;
	//UBRRL=0xA0;		//dla 2400@16MHz
	//UBRRL=207;		//dla 4800@16MHz
	//UBRRL=103;		//dla 9600@16MHz
	//UBRRL=51;			//dla 19200@16MHz
	//UBRRL=16;			//dla 57600@16MHz
	UBRRL=8;			//dla 57600@8MHz
	//UBRRL=8;			//dla 115200@16MHz
   
   UCSRB=0x98;//10011000
   UCSRC=0x86;
   
   x = 0;			   /* Flush receive buffer */
   UART_RxTail = x;
   UART_RxHead = x;
   UART_TxTail = x;
   UART_TxHead = x;
   
   uart_echo=0;
   
}
////////////////////////////////////////////////////////////////////////////////
//Wylaczanie RS
////////////////////////////////////////////////////////////////////////////////
extern void UART_disable(void)
{
	UCSRB&=~((1<<RXCIE)|(1<<TXCIE)|(1<<UDRIE)|(1<<RXEN)|(1<<TXEN));
	
}
////////////////////////////////////////////////////////////////////////////////
//Przerwanie od odebranego znaku
////////////////////////////////////////////////////////////////////////////////
SIGNAL(SIG_UART_RECV)
{
   
	static unsigned char data;
	static unsigned char tmphead;

	data = UDR; 						//przepisz rejestr do data

   	
	tmphead = ( UART_RxHead + 1 ) & UART_RX_BUFFER_MASK;
									//oblicz wskasnik bufora
	UART_RxHead = tmphead;		

	if ( tmphead == UART_RxTail )	//obsluga bledu przepelnienia bufora
	{
		UART_flush_rx_buffer();  	

	}

	else
	{
	UART_RxBuf[tmphead] = data; 	//zapisz dane w buforze odbiorczym
	}

}
////////////////////////////////////////////////////////////////////////////////
//Przerwanie od pustego rejestru nadawczego (wywolywane jest cyklicznie
//jesli dane zostaly wyslane, dlatego jest niezbedne jego wlaczanie/wylaczanie)
////////////////////////////////////////////////////////////////////////////////
SIGNAL(SIG_UART_DATA)
{
	
	static unsigned char tmptail;
	static unsigned char temp;
	
	if ( UART_TxHead != UART_TxTail )//sprawdz czy wyslano juz wszystkie dane
	{
		tmptail = ( UART_TxTail + 1 ) & UART_TX_BUFFER_MASK;
		UART_TxTail = tmptail;	  
		temp = UART_TxBuf[tmptail];	//rozpocznij transmisje
		UDR = temp;

	}
	else
	{
	  	UCSRB&=~(1<<UDRIE);			//wylacz przerwania UDRE
	}
}
////////////////////////////////////////////////////////////////////////////////
//Odbieranie danych - pobieranie z bufora
////////////////////////////////////////////////////////////////////////////////
extern unsigned char UART_receive_byte( void )
{
	unsigned char tmptail;
	
	
	while(UART_RxHead == UART_RxTail);
	
   	tmptail = ( UART_RxTail + 1 ) & UART_RX_BUFFER_MASK;
   									//pobierz dane z bufora
   	UART_RxTail = tmptail;	
   	

   
	return UART_RxBuf[tmptail]; 
}
////////////////////////////////////////////////////////////////////////////////
//Wysylanie bajtu poprzez wpisanie do bufora nadawczeo i uruchomienie przerwania
//od pustego rejestru danej do wyslania
////////////////////////////////////////////////////////////////////////////////
extern void UART_transmit_byte( unsigned char data )
{
	unsigned char tmphead;
	

	
   	tmphead = ( UART_TxHead + 1 ) & UART_TX_BUFFER_MASK; 
   									
   	while ( tmphead == UART_TxTail );
   									//czekaj na wolne miejsce w buforze
   	
   	UART_TxBuf[tmphead] = data;		//zapisz dane w buforze
   	UART_TxHead = tmphead;			

   	UCSRB|=(1<<UDRIE);				//uruchom przerwania
   	
}
////////////////////////////////////////////////////////////////////////////////
//Wysylanie bajtu kontrolnego - poza kolejnoscia w buforze.
//Funkcja ma na celu przeslanie ASAP pojedynczego bajtu, wylaczenie przerwan
//powoduje, ze natychmiast po zwolnieniu interfejsu jest do niego zaladowany bajt.
//Operacja nie jest buforowana, takze nie ma wplywu na bufor nadawczy.
////////////////////////////////////////////////////////////////////////////////
#ifdef XON_XOFF
extern void UART_transmit_control(unsigned char data)
{
	static unsigned char sreg;
	static unsigned char UDRIE_state;
	
	sreg=SREG;						//zachowanie stanu rejestru statusowego
	cli();							//wylaczenie przerwan
	
	UDRIE_state=UCSRB&(1<<UDRIE);	//zachowanie stanu bitu zezw. na przerwanie
	
	
	UCSRB&=~(1<<UDRIE);				//wylaczenie przerwania
	while(!(UCSRA&(1<<UDRE)));		//czekanie na zwolnienie interfejsu
	
	UDR=data;						//zapisanie rejestru danych
	
	UCSRB|=UDRIE_state;				//przywrocenie stanu bitu zezw. na przerwanie
	
	SREG=sreg;						//przywrocenie stanu rejestru statusowego
}
#endif
////////////////////////////////////////////////////////////////////////////////
//Sprawdzenie, czy w buforze rx sa dane
////////////////////////////////////////////////////////////////////////////////
extern unsigned char UART_data_in_rx_buffer( void )
{

	
   return ( UART_RxHead != UART_RxTail );  
   									//zwroc zero jesli bufor wejsciowy pusty
}
////////////////////////////////////////////////////////////////////////////////
//Sprawdzenie, czy sa dane do przeslania w buforze tx
////////////////////////////////////////////////////////////////////////////////
extern unsigned char UART_data_in_tx_buffer( void )
{
   return ( UART_TxHead != UART_TxTail ); 
   									//zwroc zero jesli bufor wyjsciowy pusty
}
////////////////////////////////////////////////////////////////////////////////
//Odczytywanie ilosci bajtow w buforach
////////////////////////////////////////////////////////////////////////////////
extern unsigned char UART_bytes_in_rx_buffer(void)
{
	unsigned char temp;
	unsigned char sreg;
	sreg=SREG;
	cli();
	
	if(UART_RxHead >= UART_RxTail)
	{
		temp=UART_RxHead-UART_RxTail;
		SREG=sreg;
		return(temp);
	}
	else
	{
		temp=UART_RX_BUFFER_SIZE-UART_RxTail+UART_RxHead;
		SREG=sreg;
		return(temp);
	}
	
}
extern unsigned char UART_bytes_in_tx_buffer(void)
{
	unsigned char temp;
	unsigned char sreg;
	sreg=SREG;
	cli();
	
	if(UART_TxHead >= UART_TxTail)
	{
		temp=UART_TxHead-UART_TxTail;
		SREG=sreg;
		return(temp);
	}
	else
	{
		temp=UART_TX_BUFFER_SIZE-UART_TxTail+UART_TxHead;
		SREG=sreg;
		return(temp);
	}
}
////////////////////////////////////////////////////////////////////////////////
//Czyszczenie buforow
////////////////////////////////////////////////////////////////////////////////
extern void UART_flush_tx_buffer(void)
{
	unsigned char sreg;				//zachowanie stanu rejestru statusowego
	sreg=SREG;
	cli();
		UART_TxHead=0;
		UART_TxTail=0;
	SREG=sreg;
}
extern void UART_flush_rx_buffer(void)
{
	unsigned char sreg;
	sreg=SREG;
	cli();
		UART_RxHead=0;
		UART_RxTail=0;
	SREG=sreg;
}
////////////////////////////////////////////////////////////////////////////////
//Przejscie do nowej linii i dodanie znaku konca stringa (konwencja DOS - \r\n)
////////////////////////////////////////////////////////////////////////////////
extern void UART_return(void)//przejscie do nowej linii oraz zakonczenie stringa w terminalu
{
UART_transmit_byte('\0');
UART_transmit_byte('\r');
UART_transmit_byte('\n');
}
////////////////////////////////////////////////////////////////////////////////
//Wypisywanie stringa na terminal (wlacznie ze znakiem konca stringa)
////////////////////////////////////////////////////////////////////////////////
extern unsigned char UART_print(unsigned char *tekst)//wypisywanie na terminal
{
unsigned char i=0;


for(;;)
{
	UART_transmit_byte(*(tekst+i));
	if(!(*(tekst+i)))	break;
	i++;
}


return i;
}

////////////////////////////////////////////////////////////////////////////////
//Odczytywanie stringow lub linii z rsa
////////////////////////////////////////////////////////////////////////////////
extern unsigned char UART_gets(unsigned char *buf,unsigned char ile)
{
unsigned char i=0;

while(i<ile)
{
	*(buf+i)=UART_receive_byte();

	if((*(buf+i)=='\r')||(*(buf+i)=='\n')||(*(buf+i)=='\0'))
		{
			*(buf+i)='\0';
			return(i+1);	//zwroc ilosc znakow wraz ze znakiem \0
		}
	else
		{
			i++;//przestawienie wskaznika na nowa pozyzje tablicy
		}
}

*(buf+i-1)='\0';
return(i);							//zwracamy liczbe odebranych bajtow
}


#ifdef UART_ANSI_TERMINAL
	extern void UART_goto(unsigned char row,unsigned char col)
	{
	SEND_ESC;
	UART_print(itoa(row,buf,10));
	UART_transmit_byte(';');
	UART_print(itoa(col,buf,10));
	UART_transmit_byte('H');
	}
	
	extern void UART_set_font( unsigned char mode,unsigned char fg_color, unsigned char bg_color )
	{
		SEND_ESC;
		UART_print(itoa(mode,buf,10));				   
		UART_transmit_byte(';');
		UART_print(itoa(fg_color,buf,10));				   
		UART_transmit_byte(';');
		UART_print(itoa(bg_color,buf,10));				   
		UART_transmit_byte('m');
	}
#endif//UART_ANSI_TERMINAL
///////////////////////////////////////////////////////////////////////////////////////////
extern void UART_buffer_rx_byte(unsigned char data)
{
	//unsigned char sreg;
	
	
	unsigned char tmphead;
	tmphead = ( UART_RxHead + 1 ) & UART_RX_BUFFER_MASK;
	UART_RxHead = tmphead;/* Store new index */
	
	if ( tmphead == UART_RxTail )//obsluga bledu przepelnienia bufora
	{
		UART_flush_rx_buffer(); 
	}

	else
	{
	UART_RxBuf[tmphead] = data; 	//zapisz dane w buforze odbiorczym
	}
}
////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
extern unsigned char UART_buffer_rx_block(unsigned char *tekst)//wypisywanie na terminal
{
unsigned char i=0;
while(/*(*(tekst+i)!='\r')&&*/(*(tekst+i)!='\0'))
{
   UART_buffer_rx_byte(*(tekst+i));
   i++;
}
UART_buffer_rx_byte(*(tekst+i));

return i;
}
////////////////////////////////////////////////////////////////////////////////
//Przesylanie danych binarnych
////////////////////////////////////////////////////////////////////////////////
extern void UART_send_block(const void *var, unsigned char cnt)
{
	unsigned char* ptr;
	ptr=(unsigned char *)var;
					
	for(unsigned int i=0;i<cnt;i++)
	{
		UART_transmit_byte(*(ptr));
		ptr++;
	}
}
////////////////////////////////////////////////////////////////////////////////
//Odbieranie danych binarnych
////////////////////////////////////////////////////////////////////////////////
extern void UART_receive_block(void *var, unsigned char cnt)
{
	unsigned char* ptr;
	ptr=(unsigned char *)var;
					
	for(unsigned int i=0;i<cnt;i++)
	{
		*(ptr)=UART_receive_byte();
		ptr++;
	}
}
